import math
import imp
import datetime
imp.load_source("util","ScriptEditor/util.py")  
from util import *
imp.load_source("boundary","ScriptEditor/boundary.py")  
from boundary import *
import sys

translator.install()

def getBoundaryAnalysisSize(analysis):
	if "length" in analysis:
		return analysis["length"]
	if "area" in analysis:
		return analysis["area"]
	return 0.0
	
def getYearDuration():
	num28DaysSeconds = 2419200.0;
	num30DaysSeconds = 2592000.0;
	num31DaysSeconds = 2678400.0;

	return num28DaysSeconds + num30DaysSeconds * 4.0 + num31DaysSeconds * 7.0
	
def getMonthDuration(month):

	monthMod = month % 12

	num28DaysSeconds = 2419200.0;
	num30DaysSeconds = 2592000.0;
	num31DaysSeconds = 2678400.0;
	
	if monthMod == 0 or \
		monthMod == 2 or \
		monthMod == 4 or \
		monthMod == 6 or \
		monthMod == 7 or \
		monthMod == 9 or \
		monthMod == 11:
		return num31DaysSeconds
		
	if monthMod == 3 or \
		monthMod == 5 or \
		monthMod == 8 or \
		monthMod == 10:
		return num30DaysSeconds
		
	return num28DaysSeconds

def getMonthTime(month):
	if month < 0:
		return 0.0
	
	year = int(month / 12)
	monthMod = month % 12
		
	num28DaysSeconds = 2419200.0;
	num30DaysSeconds = 2592000.0;
	num31DaysSeconds = 2678400.0;
	
	seconds = year * 31536000.0
	if monthMod < 1:
		return seconds
	seconds = seconds + num31DaysSeconds
	if monthMod < 2:
		return seconds
	seconds = seconds + num28DaysSeconds
	if monthMod < 3:
		return seconds
	seconds = seconds + num31DaysSeconds
	if monthMod < 4:
		return seconds
	seconds = seconds + num30DaysSeconds
	if monthMod < 5:
		return seconds
	seconds = seconds + num31DaysSeconds
	if monthMod < 6:
		return seconds
	seconds = seconds + num30DaysSeconds
	if monthMod < 7:
		return seconds
	seconds = seconds + num31DaysSeconds
	if monthMod < 8:
		return seconds
	seconds = seconds + num31DaysSeconds
	if monthMod < 9:
		return seconds
	seconds = seconds + num30DaysSeconds
	if monthMod < 10:
		return seconds
	seconds = seconds + num31DaysSeconds
	if monthMod < 11:
		return seconds
	seconds = seconds + num30DaysSeconds
	if monthMod < 12:
		return seconds
	seconds = seconds + num31DaysSeconds
	
	return seconds
	
def getMonth(time):
	currentTime = time;
	for i in range(24):
		currentTime = currentTime + getMonthDuration(i);
		if currentTime > time:
			return i
	return 23
	#for i in range(12):
		#if getMonthTime(i + 1) > time:
		#	return i
	#return 11
	
def getMonthName(month):
	monthMod = month % 12
	if monthMod == 0:
		return _("January")
	if monthMod == 1:
		return _("February")
	if monthMod == 2:
		return _("March")
	if monthMod == 3:
		return _("April")
	if monthMod == 4:
		return _("May")
	if monthMod == 5:
		return _("June")
	if monthMod == 6:
		return _("July")
	if monthMod == 7:
		return _("August")
	if monthMod == 8:
		return _("September")
	if monthMod == 9:
		return _("October")
	if monthMod == 10:
		return _("November")
	if monthMod == 11:
		return _("December")
	return ""

def createAnalysisCurve(bind, color, legend):
	newAnalysisCurve = dict()
	newAnalysisCurve["points"] = list()
	newAnalysisCurve["color"] = color
	newAnalysisCurve["bind"] = bind
	newAnalysisCurve["integral"] = 0.0
	newAnalysisLegend = dict()
	newAnalysisLegend["value"] = legend
	newAnalysisLegend["color"] = color
	return newAnalysisCurve, newAnalysisLegend

def createBoundaryDynamicTemperatureAnalysis(title):
	newAnalysisMinCurve, newAnalysisMinLegend = createAnalysisCurve("min_t", "blue", _("Min. T"))
	newAnalysisMaxCurve, newAnalysisMaxLegend = createAnalysisCurve("max_t", "red", _("Max. T"))
	newAnalysisAvCurve, newAnalysisAvLegend = createAnalysisCurve("av_t", "green", _("Av. T"))

	newAnalysisCurves = list()
	newAnalysisCurves.append(newAnalysisMinCurve)
	newAnalysisCurves.append(newAnalysisMaxCurve)
	newAnalysisCurves.append(newAnalysisAvCurve)
	newAnalysisLegends = list()
	newAnalysisLegends.append(newAnalysisMinLegend)
	newAnalysisLegends.append(newAnalysisMaxLegend)
	newAnalysisLegends.append(newAnalysisAvLegend)
	newAnalysis = dict()
	newAnalysis["title"] = title
	newAnalysis["font_height"] = 8
	newAnalysis["curves"] = newAnalysisCurves
	newAnalysis["legends"] = newAnalysisLegends
	newAnalysis["unit"] = "°C"
	newAnalysis["__seed__"] = int(datetime.datetime.now().microsecond)
	return newAnalysis

def createBoundaryDynamicFlowAnalysis(title):
	newAnalysisQCurve, newAnalysisQLegend = createAnalysisCurve("Q", "red", _("Q"))

	newAnalysisCurves = list()
	newAnalysisCurves.append(newAnalysisQCurve)
	newAnalysisLegends = list()
	newAnalysisLegends.append(newAnalysisQLegend)
	newAnalysis = dict()
	newAnalysis["title"] = title
	newAnalysis["font_height"] = 8
	newAnalysis["curves"] = newAnalysisCurves
	newAnalysis["legends"] = newAnalysisLegends
	newAnalysis["unit"] = "W/m"
	newAnalysis["__seed__"] = int(datetime.datetime.now().microsecond)
	return newAnalysis
	
def createBoundaryDynamicCondensationAnalysis(title, simulation_3d):
	if not simulation_3d:
		dimensionBind = "length"
		dimensionUnit = "mm"
	else:
		dimensionBind = "area"
		dimensionUnit = "m²"

	newAnalysisDewCurve, newAnalysisDewLegend = createAnalysisCurve("dew_" + dimensionBind, "blue", _("Surf. Cond."))
	newAnalysisMoldCurve, newAnalysisMoldLegend = createAnalysisCurve("mold_" + dimensionBind, "green", _("Mold"))

	newAnalysisCurves = list()
	newAnalysisCurves.append(newAnalysisDewCurve)
	newAnalysisCurves.append(newAnalysisMoldCurve)
	newAnalysisLegends = list()
	newAnalysisLegends.append(newAnalysisDewLegend)
	newAnalysisLegends.append(newAnalysisMoldLegend)
	newAnalysis = dict()
	newAnalysis["title"] = title
	newAnalysis["font_height"] = 8
	newAnalysis["curves"] = newAnalysisCurves
	newAnalysis["legends"] = newAnalysisLegends
	newAnalysis["unit"] = dimensionUnit
	newAnalysis["__seed__"] = int(datetime.datetime.now().microsecond)
	return newAnalysis
	
def createBoundaryDynamicAnalysis(curveBind, title, color, legend, unit):
	newAnalysisCurve, newAnalysisLegend = createAnalysisCurve(curveBind, color, legend)

	newAnalysisCurves = list()
	newAnalysisCurves.append(newAnalysisCurve)
	newAnalysisLegends = list()
	newAnalysisLegends.append(newAnalysisLegend)
	newAnalysis = dict()
	newAnalysis["title"] = title
	newAnalysis["font_height"] = 8
	newAnalysis["curves"] = newAnalysisCurves
	newAnalysis["legends"] = newAnalysisLegends
	newAnalysis["unit"] = unit
	newAnalysis["__seed__"] = int(datetime.datetime.now().microsecond)	
	return newAnalysis
	
def addBoundaryDynamicTemperatureAnalysisStep(time,\
											boundaryConditions,\
											boundaryGroups,\
											stepBoundaryAnalysis,\
											analysis,\
											groupAnalysis):
	while len(analysis) < len(stepBoundaryAnalysis):
		analysis.append(createBoundaryDynamicTemperatureAnalysis(boundaryConditions[len(analysis)]["name"] + " " + _("dynamic analysis")))
	while len(groupAnalysis) < len(boundaryGroups):
		boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions)
		groupAnalysis.append(createBoundaryDynamicTemperatureAnalysis(boundaryGroupNames[len(groupAnalysis)] + " " + _("dynamic analysis")))
	
	boundaryGroupID = 0
	for boundaryGroup in boundaryGroups:
		groupSize = 0.0
		groupMinT = 0.0
		groupMaxT = 0.0
		groupAvT = 0.0
		for boundaryID in boundaryGroup:
			boundarySize = getBoundaryAnalysisSize(stepBoundaryAnalysis[boundaryID])
			boundaryMinT = stepBoundaryAnalysis[boundaryID]["min_t"]
			boundaryMaxT = stepBoundaryAnalysis[boundaryID]["max_t"]
			boundaryAvT = stepBoundaryAnalysis[boundaryID]["av_t"]
			
			analysis[boundaryID]["curves"][0]["points"].append([time, boundaryMinT])
			analysis[boundaryID]["curves"][1]["points"].append([time, boundaryMaxT])
			analysis[boundaryID]["curves"][2]["points"].append([time, boundaryAvT])
			groupMinT = boundaryMinT * boundarySize
			groupMaxT = boundaryMaxT * boundarySize
			groupAvT = boundaryAvT * boundarySize
			groupSize = groupSize + boundarySize
		if groupSize > 0.0:
			groupMinT = groupMinT / groupSize
			groupMaxT = groupMaxT / groupSize
			groupAvT = groupAvT / groupSize
		groupAnalysis[boundaryGroupID]["curves"][0]["points"].append([time, groupMinT])
		groupAnalysis[boundaryGroupID]["curves"][1]["points"].append([time, groupMaxT])
		groupAnalysis[boundaryGroupID]["curves"][2]["points"].append([time, groupAvT])
		boundaryGroupID = boundaryGroupID + 1

def addBoundaryDynamicFlowAnalysisStep(time,\
										timeStep,\
										boundaryConditions,\
										boundaryGroups,\
										stepBoundaryAnalysis,\
										analysis,\
										groupAnalysis):
	while len(analysis) < len(stepBoundaryAnalysis):
		analysis.append(createBoundaryDynamicFlowAnalysis(boundaryConditions[len(analysis)]["name"] + " " + _("dynamic analysis")))
	while len(groupAnalysis) < len(boundaryGroups):
		boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions)
		groupAnalysis.append(createBoundaryDynamicFlowAnalysis(boundaryGroupNames[len(groupAnalysis)] + " " + _("dynamic analysis")))
	
	boundaryGroupID = 0	
	for boundaryGroup in boundaryGroups:
		groupQ = 0.0
		for boundaryID in boundaryGroup:
			boundaryLength = getBoundaryAnalysisSize(stepBoundaryAnalysis[boundaryID])
			boundaryQ = stepBoundaryAnalysis[boundaryID]["Q"]
			analysis[boundaryID]["curves"][0]["points"].append([time, boundaryQ])
			analysis[boundaryID]["curves"][0]["integral"] = analysis[boundaryID]["curves"][0]["integral"] + boundaryQ * timeStep
			groupQ = groupQ + boundaryQ
		groupAnalysis[boundaryGroupID]["curves"][0]["points"].append([time, groupQ])
		groupAnalysis[boundaryGroupID]["curves"][0]["integral"] = groupAnalysis[boundaryGroupID]["curves"][0]["integral"] + groupQ * timeStep
		boundaryGroupID = boundaryGroupID + 1
		
def addBoundaryDynamicCondensationAnalysisStep(time,\
												boundaryConditions,\
												boundaryGroups,\
												stepBoundaryAnalysis,\
												analysis,\
												groupAnalysis,\
												simulation_3d):
	while len(analysis) < len(stepBoundaryAnalysis):
		analysis.append(createBoundaryDynamicCondensationAnalysis(boundaryConditions[len(analysis)]["name"] + " " + _("dynamic analysis"), simulation_3d))
	while len(groupAnalysis) < len(boundaryGroups):
		boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions)
		groupAnalysis.append(createBoundaryDynamicCondensationAnalysis(boundaryGroupNames[len(groupAnalysis)] + " " + _("dynamic analysis"), simulation_3d))
	
	boundaryGroupID = 0
	for boundaryGroup in boundaryGroups:
		groupDew = 0.0
		groupMold = 0.0
		for boundaryID in boundaryGroup:
			dimensionBind = getTFAnalysisDimensionBind(stepBoundaryAnalysis[boundaryID])
			boundaryDew = stepBoundaryAnalysis[boundaryID]["dew_" + dimensionBind]
			boundaryMold = stepBoundaryAnalysis[boundaryID]["mold_" + dimensionBind]
			
			analysis[boundaryID]["curves"][0]["points"].append([time, boundaryDew])
			analysis[boundaryID]["curves"][1]["points"].append([time, boundaryMold])
			groupDew = groupDew + boundaryDew
			groupMold = groupMold + boundaryMold
		groupAnalysis[boundaryGroupID]["curves"][0]["points"].append([time, groupDew])
		groupAnalysis[boundaryGroupID]["curves"][1]["points"].append([time, groupMold])
		boundaryGroupID = boundaryGroupID + 1
		
def addBoundariesCurvesStep(boundaryConditions,\
							boundariesAtSimulationTime,\
							time,\
							stepBoundaryAnalysis,\
							analysis):
	for boundaryCondition in boundaryConditions:
		boundaryID = boundaryCondition["id"]
		boundaryAnalysis = stepBoundaryAnalysis[boundaryID]
		if getBoundaryAnalysisSize(boundaryAnalysis) == 0.0:
			continue
		analysisBinds = list()
		analysisTitles = list()
		analysisColors = list()
		analysisLegends = list()
		analysisUnits = list()
		if boundaryCondition["type"] == "environment_air":
			if boundaryCondition["air_temperature_type"] != "fixed":
				analysisBinds.append("temperature")
				analysisTitles.append(boundaryCondition["name"] + " " + _("temperature"))
				analysisColors.append("blue")
				analysisLegends.append(_("Temperature"))
				analysisUnits.append("°C")

		if boundaryCondition["type"] == "environment_air" or boundaryCondition["type"] == "flow_rate":
			if "flow_rate_type" in boundaryCondition and boundaryCondition["flow_rate_type"] != "fixed":
				analysisBinds.append("flow_rate")
				analysisTitles.append(boundaryCondition["name"] + " " + _("flow rate"))
				analysisColors.append("blue")
				analysisLegends.append(_("Flow rate"))
				analysisUnits.append("W/m²")
				
		if boundaryCondition["type"] == "fixed_temperature":
			if boundaryCondition["temperature_type"] != "fixed":
				analysisBinds.append("temperature")
				analysisTitles.append(boundaryCondition["name"] + " " + _("temperature"))
				analysisColors.append("blue")
				analysisLegends.append(_("Temperature"))
				analysisUnits.append("°C")				

		if boundaryCondition["humidity_type"] != "fixed":
			analysisBinds.append("humidity")
			analysisTitles.append(boundaryCondition["name"] + " " + _("humidity"))
			analysisColors.append("blue")
			analysisLegends.append(_("humidity"))
			analysisUnits.append("%")					
		
		for analysisID in range(len(analysisBinds)):
			dynamicAnalysis = False
			for boundaryDynamicAnalysis in analysis:
				if boundaryDynamicAnalysis["bind"] == analysisBinds[analysisID] + str(boundaryID):
					dynamicAnalysis = boundaryDynamicAnalysis
					break
			if dynamicAnalysis == False:
				dynamicAnalysis = createBoundaryDynamicAnalysis(analysisBinds[analysisID],\
																analysisTitles[analysisID],\
																analysisColors[analysisID],\
																analysisLegends[analysisID],\
																analysisUnits[analysisID])
				dynamicAnalysis["bind"] = analysisBinds[analysisID] + str(boundaryID)
				analysis.append(dynamicAnalysis)
			dynamicAnalysis["curves"][0]["points"].append([time, boundariesAtSimulationTime[boundaryID][analysisBinds[analysisID]]])
		
def addSingleDynamicBoundaryPlot(transientSimulationSettings,\
								name,\
								bind,\
								analysis,\
								simulationOutputBoundary,\
								simulationOutputBoundaryDescriptors,\
								timeUnitType = "time"):
	dynamicBoundaryAnalysisDescriptor = dict()
	dynamicBoundaryAnalysisDescriptor["type"] = "plot"
	dynamicBoundaryAnalysisDescriptor["bind"] = bind
	dynamicBoundaryAnalysisDescriptor["__tag__"] = bind
	dynamicBoundaryAnalysisDescriptor["name"] = name
		
	# Search for min and max points
	minTime = sys.float_info.max
	maxTime = -sys.float_info.max
	minY = sys.float_info.max
	maxY = -sys.float_info.max
	analysisValues = list()
	for curve in analysis["curves"]:
		avY = 0.0
		for point in curve["points"]:
			if point[1] < minY:
				minY = point[1]
				minTime = point[0]
			if point[1] > maxY:
				maxY = point[1]
				maxTime = point[0]
			avY = avY + point[1] / len(curve["points"])
		minValue = dict()
		minValue["value"] = minY
		minValue["position"] = [minTime, minY]
		minValue["unit"] = analysis["unit"]
		if minValue["unit"] == "%":
			minValue["format"] = "%.2f"
		else:
			minValue["format"] = "%.2g"
		minValue["color"] = "black"
		maxValue = dict()
		maxValue["value"] = maxY
		maxValue["position"] = [maxTime, maxY]
		maxValue["unit"] = analysis["unit"]
		if maxValue["unit"] == "%":
			maxValue["format"] = "%.2f"
		else:
			maxValue["format"] = "%.2g"
		maxValue["color"] = "black"
		
		curve["min_y"] = minY
		curve["max_y"] = maxY
		curve["av_y"] = avY	
		analysisValues.append(minValue)
		analysisValues.append(maxValue)
		
		if curve["bind"] == "Q" and "integral" in curve:
			analysis["legends"][0]["value"] += " (" + \
				'%(integral)G' % {"integral": curve["integral"]} + \
				" J/m)";
		
	analysis["values"] = analysisValues

	minTimeAxis = sys.float_info.max
	maxTimeAxis = -sys.float_info.max
	
	for curve in analysis["curves"]:
		if len(curve["points"]) > 0:
			if curve["points"][0][0] < minTimeAxis:
				minTimeAxis = curve["points"][0][0]
			if curve["points"][-1][0] > maxTimeAxis:
				maxTimeAxis = curve["points"][-1][0]

	timeAxisExtent = maxTimeAxis - minTimeAxis
	
	minuteSeconds = 60.0
	hourSeconds = minuteSeconds * 60.0
	daySeconds = hourSeconds * 24.0
	avMonthSeconds = daySeconds * 30.0
	yearSeconds = daySeconds * 365.0
	
	if(timeUnitType == "time"):		
		timeUnit = transientSimulationSettings["graphs_time_unit"]
		relativeTimeUnit = transientSimulationSettings["graphs_time_unit_relative"]
		autoTimeUnit = timeUnit == "automatic"

		axisLegendInterval = yearSeconds
		timeUnitName = _("years")
		valueMultiplier = 1
		if (timeAxisExtent < minuteSeconds * 1.5 and autoTimeUnit) or timeUnit == "second":
			axisLegendInterval = 1.0
			timeUnitName = _("seconds")
		elif (timeAxisExtent < hourSeconds * 1.5 and autoTimeUnit) or timeUnit == "minute":
			axisLegendInterval = minuteSeconds
			timeUnitName = _("minutes")
		elif (timeAxisExtent < daySeconds * 3.0 and autoTimeUnit) or timeUnit == "hour":
			axisLegendInterval = hourSeconds
			timeUnitName = _("hours")
		elif timeUnit == "6_hours":
			valueMultiplier = 6.0
			axisLegendInterval = hourSeconds * valueMultiplier
			timeUnitName = _("hours")
		elif (timeAxisExtent < avMonthSeconds * 3.0 and autoTimeUnit) or timeUnit == "day":
			axisLegendInterval = daySeconds
			timeUnitName = _("days")
		elif (timeAxisExtent < yearSeconds * 4.0 and autoTimeUnit) or timeUnit == "month":
			axisLegendInterval = avMonthSeconds
			timeUnitName = _("months")
			
		while(timeAxisExtent / axisLegendInterval > 100):
			axisLegendInterval = axisLegendInterval * 2
			valueMultiplier = valueMultiplier * 2
	else:
		timeUnit = "m"
		timeUnitName = timeUnit
		relativeTimeUnit = False
		axisLegendInterval = timeAxisExtent / 10.0
		valueMultiplier = 0.1
	
	analysisAxisLegends = list()
	axisLegendColor = list();
	axisLegendColor.append(0.5)
	axisLegendColor.append(0.5)
	axisLegendColor.append(0.5)
	
	if axisLegendInterval == avMonthSeconds and timeUnitType == "time":
		startAxisYear = int(minTimeAxis / yearSeconds)
		startAxisYearSeconds = startAxisYear * yearSeconds
		if startAxisYearSeconds > minTimeAxis:
			startAxisYear = startAxisYear - 1
			startAxisYearSeconds = startAxisYearSeconds - yearSeconds
		yearOffset = minTimeAxis - startAxisYearSeconds
		startMonth = 0
		while getMonthTime(startMonth) < yearOffset and startMonth < 11:
			startMonth = startMonth + 1
		currentMonth = startMonth
		if currentMonth > 0:
			currentMonth = currentMonth - 1
		startAxisValue = yearOffset + getMonthTime(currentMonth)
		axisValue = startAxisValue
		while axisValue <= maxTimeAxis:
			axisLegend = dict()
			axisLegend["value"] = getMonthName(currentMonth)
			axisLegend["axis_value"] = axisValue
			axisLegend["axis"] = 0
			axisLegend["color"] = axisLegendColor

			currentMonthTime = getMonthTime(currentMonth)
			currentMonth = currentMonth + 1
			if currentMonth > 11:
				currentMonth = 0
				axisValue = axisValue + yearSeconds - currentMonthTime
			else:
				axisValue = axisValue + getMonthTime(currentMonth) - currentMonthTime
			analysisAxisLegends.append(axisLegend)		
	else:
		startAxisValueInt = int(minTimeAxis / axisLegendInterval)
		#startAxisValue = startAxisValueInt * axisLegendInterval
		axisValue = minTimeAxis#startAxisValue
		axisValueInt = startAxisValueInt
		while axisValue <= maxTimeAxis:
			axisLegend = dict()
			if relativeTimeUnit:
				adaptedAxisValueInt = axisValueInt - startAxisValueInt
			else:
				adaptedAxisValueInt = axisValueInt

			axisLegend["value"] = str(adaptedAxisValueInt * valueMultiplier)
			if(axisValueInt == startAxisValueInt):
				axisLegend["value"] = axisLegend["value"] + " " + timeUnitName
			axisLegend["axis_value"] = axisValue
			axisLegend["axis"] = 0
			axisLegend["color"] = axisLegendColor
			analysisAxisLegends.append(axisLegend)
			axisValue = axisValue + axisLegendInterval
			axisValueInt = axisValueInt + 1
			
	analysis["axis_legends"] = analysisAxisLegends
		
	simulationOutputBoundary[bind] = analysis
	simulationOutputBoundaryDescriptors.append(dynamicBoundaryAnalysisDescriptor)

def fixBoundaryTfAnalysisForDynamic(boundaryTFAnalysis, dynamicAnalysis):
	dimensionBind = getTFAnalysisDimensionBind(boundaryTFAnalysis)
	for curve in dynamicAnalysis["curves"]:
		if curve["bind"] == "min_t":
			boundaryTFAnalysis["min_t"] = curve["min_y"]
		elif curve["bind"] == "max_t":
			boundaryTFAnalysis["max_t"] = curve["max_y"]
		elif curve["bind"] == "av_t":
			boundaryTFAnalysis["av_t"] = curve["av_y"]
		elif curve["bind"] == "dew_" + dimensionBind:
			boundaryTFAnalysis["dew_" + dimensionBind] = curve["av_y"]
		elif curve["bind"] == "mold_" + dimensionBind:
			boundaryTFAnalysis["mold_" + dimensionBind] = curve["av_y"]
		elif curve["bind"] == "Q":
			boundaryTFAnalysis["Q"] = curve["av_y"]
	
def addDynamicBoundaryAnalysis(transientSimulationSettings,\
								boundaryConditions,\
								boundaryGroups,\
								simulationOutput,\
								simulationOutputDescriptors,\
								dynamicBoundaryAnalysis,\
								dynamicBoundaryGroupAnalysis,\
								analysisBind,\
								addGroupAnalysis = True):
	
	simulationOutputAdvancedResults = simulationOutput["advanced_results"]
	simulationOutputBoundary = simulationOutputAdvancedResults["boundary"]
	simulationBoundaryTFAnalysis = simulationOutputBoundary["tf_analysis"]
	simulationOutputAdvancedResultsDescriptors = findSubDescriptors("advanced_results", simulationOutputDescriptors)
	simulationOutputBoundaryDescriptors = findSubDescriptors("boundary",  simulationOutputAdvancedResultsDescriptors)
	#simulationOutputBoundaryTFAnalysisDescriptors = findSubDescriptors("tf_analysis",  simulationOutputBoundaryDescriptors)
		
	for boundaryCondition in boundaryConditions:
		if boundaryCondition["type"] == "adiabatic":
			continue
		boundaryID = boundaryCondition["id"]
		if getBoundaryAnalysisSize(simulationBoundaryTFAnalysis[boundaryID]) == 0.0:
			continue

		addSingleDynamicBoundaryPlot(transientSimulationSettings,\
									boundaryCondition["name"] + " " + analysisBind + " " + _("dynamic analysis"),\
									analysisBind + "_d_an_" + str(boundaryID),\
									dynamicBoundaryAnalysis[boundaryID],\
									simulationOutputBoundary,\
									simulationOutputBoundaryDescriptors)
										
		fixBoundaryTfAnalysisForDynamic(simulationBoundaryTFAnalysis[boundaryID],
										dynamicBoundaryAnalysis[boundaryID])

	if len(boundaryGroups) != countUsedBoundaries(simulationBoundaryTFAnalysis) and addGroupAnalysis:
		boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions)
		groupID = 0
		for boundaryGroup in boundaryGroups:
			addSingleDynamicBoundaryPlot(transientSimulationSettings,\
											boundaryGroupNames[groupID] + " " + analysisBind + " " + _("dynamic analysis"),\
											analysisBind + "_d_g_an_" + str(groupID),\
											dynamicBoundaryGroupAnalysis[groupID],\
											simulationOutputBoundary,\
											simulationOutputBoundaryDescriptors)
			groupID = groupID + 1
			
# Add boundary conditions curves			
def addDynamicBoundaryCurves(transientSimulationSettings,\
							simulationOutput,\
							simulationOutputDescriptors,\
							dynamicBoundaryCurves):
	
	simulationOutputAdvancedResults = simulationOutput["advanced_results"]
	simulationOutputBoundary = simulationOutputAdvancedResults["boundary"]
	simulationOutputAdvancedResultsDescriptors = findSubDescriptors("advanced_results", simulationOutputDescriptors)
	simulationOutputBoundaryDescriptors = findSubDescriptors("boundary",  simulationOutputAdvancedResultsDescriptors)

	for dynamicBoundaryCurve in dynamicBoundaryCurves:
		addSingleDynamicBoundaryPlot(transientSimulationSettings,\
									dynamicBoundaryCurve["title"],\
									dynamicBoundaryCurve["bind"],\
									dynamicBoundaryCurve,\
									simulationOutputBoundary,\
									simulationOutputBoundaryDescriptors)

def addDynamicBoundaryXYZFactorCurve(factorResult,\
									curveBind,\
									curveName,\
									simulationOutputBoundary,\
									simulationOutputBoundaryDescriptors):
	if len(factorResult) > 0:
		analysis = dict()
		newAnalysisCurve, newAnalysisLegend = createAnalysisCurve(curveBind, "black", curveName)
		newAnalysisLegends = list()
		newAnalysisLegends.append(newAnalysisLegend)
		analysis["title"] = curveName
		analysis["font_height"] = 8
		analysis["legends"] = newAnalysisLegends
		analysis["__seed__"] = int(datetime.datetime.now().microsecond)
		
		analysis["curves"] = list()
		analysis["curves"].append(dict())
		analysis["curves"][0]["points"] = factorResult
		analysis["curves"][0]["bind"] = curveBind
		analysis["unit"] = "%"
		addSingleDynamicBoundaryPlot(dict(),\
									curveName,\
									curveBind,\
									analysis,\
									simulationOutputBoundary,\
									simulationOutputBoundaryDescriptors,\
									"metric")
									
def addDynamicBoundaryXYZFactorCurves(boundaryConditions,\
									simulationOutput,\
									simulationOutputDescriptors):

	simulationOutputAdvancedResults = simulationOutput["advanced_results"]
	simulationOutputBoundary = simulationOutputAdvancedResults["boundary"]
	simulationOutputAdvancedResultsDescriptors = findSubDescriptors("advanced_results", simulationOutputDescriptors)
	simulationOutputBoundaryDescriptors = findSubDescriptors("boundary",  simulationOutputAdvancedResultsDescriptors)
	
	boundaryConditionID = 0
	for boundaryConditionID in range(len(boundaryConditions)):
		addDynamicBoundaryXYZFactorCurve(simulationOutputBoundary["boundaries_temperature_x_factors"][boundaryConditionID],\
										"temp_x_factor_" + str(boundaryConditionID),\
										boundaryConditions[boundaryConditionID]["name"] + " " + _("T. X factor"),\
										simulationOutputBoundary,\
										simulationOutputBoundaryDescriptors)
		addDynamicBoundaryXYZFactorCurve(simulationOutputBoundary["boundaries_temperature_y_factors"][boundaryConditionID],\
										"temp_y_factor_" + str(boundaryConditionID),\
										boundaryConditions[boundaryConditionID]["name"] + " " + _("T. Y factor"),\
										simulationOutputBoundary,\
										simulationOutputBoundaryDescriptors)
		addDynamicBoundaryXYZFactorCurve(simulationOutputBoundary["boundaries_temperature_z_factors"][boundaryConditionID],\
										"temp_z_factor_" + str(boundaryConditionID),\
										boundaryConditions[boundaryConditionID]["name"] + " " + _("T. Z factor"),\
										simulationOutputBoundary,\
										simulationOutputBoundaryDescriptors)
		addDynamicBoundaryXYZFactorCurve(simulationOutputBoundary["boundaries_humidity_x_factors"][boundaryConditionID],\
										"humidity_x_factor_" + str(boundaryConditionID),\
										boundaryConditions[boundaryConditionID]["name"] + " " + _("H. X factor"),\
										simulationOutputBoundary,\
										simulationOutputBoundaryDescriptors)
		addDynamicBoundaryXYZFactorCurve(simulationOutputBoundary["boundaries_humidity_y_factors"][boundaryConditionID],\
										"humidity_y_factor_" + str(boundaryConditionID),\
										boundaryConditions[boundaryConditionID]["name"] + " " + _("H. Y factor"),\
										simulationOutputBoundary,\
										simulationOutputBoundaryDescriptors)
		addDynamicBoundaryXYZFactorCurve(simulationOutputBoundary["boundaries_humidity_z_factors"][boundaryConditionID],\
										"humidity_z_factor_" + str(boundaryConditionID),\
										boundaryConditions[boundaryConditionID]["name"] + " " + _("H. Z factor"),\
										simulationOutputBoundary,\
										simulationOutputBoundaryDescriptors)
										